import React, { Component } from 'react';
import styled from 'styled-components';
import { Machine } from 'xstate';

const CounterContainer = styled.div`
  display: flex;
  flex-direction: row;
  justify-content: stretch;
  align-items: center;
`;

const increment = state => ({ count: state.count + 1 });

const machine = Machine({
  initial: 'count',
  states: {
    count: {
      on: {
        COUNT: {
          count: { actions: [increment] }
        }
      }
    }
  }
});

export class Counter extends Component {
  state = {
    count: 0,
    appState: machine.initialState
  };
  send(event) {
    const nextState = machine.transition(this.state.appState, event);

    const nextExtState = nextState.actions.reduce(
      (extState, action) => action(extState),
      this.state
    );

    this.setState(nextExtState);
  }
  render() {
    return (
      <CounterContainer>
        <input readOnly type="number" value={this.state.count} />
        <button onClick={() => this.send('COUNT')}>Count</button>
      </CounterContainer>
    );
  }
}
