import React, { Component } from 'react';
import styled from 'styled-components';
import { Machine } from 'xstate';

const FlightContainer = styled.div`
  display: flex;
  flex-direction: column;
  justify-content: stretch;
  align-items: center;
`;

const increment = state => ({ count: state.count + 1 });

const machine = Machine({
  initial: 'oneWay',
  states: {
    oneWay: {},
    return: {}
  },
  on: {
    SELECT: {
      '.oneWay': { cond: (_, e) => e.value === 'oneWay' },
      '.return': { cond: (_, e) => e.value === 'return' }
    }
  }
});

const formatDate = date =>
  `${date.getFullYear()}-${date.getMonth() < 9
    ? '0' + (+date.getMonth() + 1)
    : date.getMonth() + 1}-${date.getDate() < 10
    ? '0' + date.getDate()
    : date.getDate()}`;

export class Flight extends Component {
  state = {
    firstDate: formatDate(new Date()),
    lastDate: formatDate(new Date()),
    appState: machine.initialState.value
  };
  send(event) {
    const nextState = machine.transition(this.state.appState, event);

    this.setState({ appState: nextState.value }, () => {
      const nextExtState = nextState.actions.reduce(
        (extState, action) => action(extState),
        this.state
      );

      this.setState(nextExtState);
    });
  }
  render() {
    const { appState, lastDate, firstDate } = this.state;

    return (
      <FlightContainer>
        <select
          value={appState}
          onChange={e => this.send({ type: 'SELECT', value: e.target.value })}
        >
          <option value="oneWay">One Way Flight</option>
          <option value="return">Return Flight</option>
        </select>
        <input
          type="date"
          name="firstDate"
          value={this.state.firstDate}
          onChange={e => this.setState({ firstDate: e.target.value })}
        />
        <input
          type="date"
          name="lastDate"
          value={
            appState === 'oneWay' ? this.state.firstDate : this.state.lastDate
          }
          onChange={e => this.setState({ lastDate: e.target.value })}
          disabled={appState === 'oneWay'}
        />
        <button
          disabled={
            (appState === 'return' && lastDate < firstDate) ||
            (appState === 'return' && (!firstDate || !lastDate)) ||
            !firstDate
          }
        >
          Submit
        </button>
      </FlightContainer>
    );
  }
}
