import React, { Component } from 'react';
import styled from 'styled-components';
import { Machine } from 'xstate';

const TemperatureContainer = styled.div`
  display: flex;
  flex-direction: row;
  justify-content: stretch;
  align-items: center;
`;

const convertCtoF = (_, event) => ({
  fahrenheit: +event.value * (9 / 5) + 32,
  celsius: event.value
});

const convertFtoC = (_, event) => ({
  celsius: (+event.value - 32) * (5 / 9),
  fahrenheit: event.value
});

const machine = Machine({
  initial: 'temp',
  states: {
    temp: {
      on: {
        CELSIUS: {
          temp: { actions: [convertCtoF] }
        },
        FAHRENHEIT: {
          temp: { actions: [convertFtoC] }
        }
      }
    }
  }
});

export class Temperature extends Component {
  state = {
    celsius: '',
    fahrenheit: '',
    appState: machine.initialState
  };
  send(event) {
    const nextState = machine.transition(this.state.appState, event);

    const nextExtState = nextState.actions.reduce(
      (extState, action) => action(extState, event),
      this.state
    );

    this.setState(nextExtState);
  }
  render() {
    return (
      <TemperatureContainer>
        <input
          type="number"
          value={this.state.celsius}
          onChange={e => this.send({ type: 'CELSIUS', value: e.target.value })}
        />
        <span>Celsius = </span>
        <input
          type="number"
          value={this.state.fahrenheit}
          onChange={e =>
            this.send({ type: 'FAHRENHEIT', value: e.target.value })}
        />
        <span>Fahrenheit</span>
      </TemperatureContainer>
    );
  }
}
