<p align="center">
  <a href="https://next.gatsbyjs.org">
    <img alt="Gatsby" src="https://www.gatsbyjs.org/monogram.svg" width="60" />
  </a>
</p>
<h1 align="center">
  Gatsby v2
</h1>

<h3 align="center">
  ⚛️ 📄 :rocket:
</h3>
<p align="center">
  <strong>Blazing fast modern site generator for React</strong><br>
  Go beyond static sites: build blogs, ecommerce sites, full-blown apps, and more with Gatsby.
</p>
<p align="center">
  <a href="https://github.com/gatsbyjs/gatsby/blob/master/LICENSE">
    <img src="https://img.shields.io/badge/license-MIT-blue.svg" alt="Gatsby is released under the MIT license." />
  </a>
  <a href="https://travis-ci.org/gatsbyjs/gatsby">
    <img src="https://travis-ci.org/gatsbyjs/gatsby.svg?branch=master" alt="Current TravisCI build status." />
  </a>
  <a href="https://www.npmjs.org/package/gatsby">
    <img src="https://img.shields.io/npm/v/gatsby.svg?style=flat-square" alt="Current npm package version." />
  </a>
  <a href="https://npmcharts.com/compare/gatsby?minimal=true">
    <img src="https://img.shields.io/npm/dm/gatsby.svg" alt="Downloads per month on npm." />
  </a>
  <a href="https://next.gatsbyjs.org/docs/how-to-submit-a-pr/">
    <img src="https://img.shields.io/badge/PRs-welcome-brightgreen.svg" alt="PRs welcome!" />
  </a>
</p>

<h3 align="center">
  <a href="https://next.gatsbyjs.org/docs/">Quickstart</a>
  <span> · </span>
  <a href="https://next.gatsbyjs.org/tutorial/">Tutorial</a>
  <span> · </span>
  <a href="https://next.gatsbyjs.org/plugins/">Plugins</a>
  <span> · </span>
  <a href="https://next.gatsbyjs.org/docs/gatsby-starters/">Starters</a>
  <span> · </span>
  <a href="https://next.gatsbyjs.org/showcase/">Showcase</a>
  <span> · </span>
  <a href="https://next.gatsbyjs.org/docs/how-to-contribute/">Contribute</a>
  <span> · </span>
  Support: <a href="https://spectrum.chat/gatsby-js">Spectrum</a>
  <span> & </span>
  <a href="https://discord.gg/0ZcbPKXt5bVoxkfV">Discord</a>
</h3>

Gatsby is a modern framework for blazing fast websites.

- **Go Beyond Static Websites.** Get all the benefits of static websites with none of the
  limitations. Gatsby sites are fully functional React apps, so you can create high-quality,
  dynamic web apps, from blogs to ecommerce sites to user dashboards.

- **Use a Modern Stack for Every Site.** No matter where the data comes from, Gatsby sites are
  built using React and GraphQL. Build a uniform workflow for you and your team, regardless of
  whether the data is coming from the same back-end.

- **Load Data From Anywhere.** Gatsby pulls in data from any data source, whether it’s Markdown
  files, a headless CMS like Contentful or WordPress, or a REST or GraphQL API. Use source plugins
  to load your data, then develop using Gatsby’s uniform GraphQL interface.

- **Performance Is Baked In.** Ace your performance audits by default. Gatsby automates code
  splitting, image optimization, inlining critical styles, lazy-loading and prefetching resources,
  and more to ensure your site is fast — no manual tuning required.

- **Host at Scale for Pennies.** Gatsby sites don’t require servers, so you can host your entire
  site on a CDN for a fraction of the cost of a server-rendered site. Many Gatsby sites can be
  hosted entirely free on services like GitHub Pages and Netlify.

[**Learn how to use Gatsby for your next project.**](https://next.gatsbyjs.org/docs/)

## What’s In This Document

- [Get Up and Running in 5 Minutes](#-get-up-and-running-in-5-minutes)
- [Learning Gatsby](#-learning-gatsby)
- [Migration Guides](#-migration-guides)
- [How to Contribute](#-how-to-contribute)
- [Thanks to Our Contributors and Sponsors](#-thanks-to-our-contributors-and-sponsors)

## 🚀 Get Up and Running in 5 Minutes

You can get a new Gatsby site up and running on your local dev environment in 5 minutes with these three steps:

1.  **Create a Gatsby site from a Gatsby starter.**

    Get your Gatsby blog set up in a single command:

    ```sh
    # install the Gatsby CLI globally
    npm install -g gatsby-cli

    # create a new Gatsby site using the default starter
    gatsby new my-blazing-fast-site
    ```

2.  **Start the site in `develop` mode.**

    Next, move into your new site’s directory and start it up:

    ```sh
    cd my-blazing-fast-site/
    gatsby develop
    ```

3.  **Open the source code and start editing!**

    Your site is now running at `http://localhost:8000`. Open the `my-blazing-fast-site` directory in your code editor of choice and edit `src/pages/index.js`. Save your changes and the browser will update in real time!

At this point, you’ve got a fully functional Gatsby website. For additional information on how you can customize your Gatsby site, see our [plugins](https://next.gatsbyjs.org/plugins/) and [the official tutorial](https://next.gatsbyjs.org/tutorial/).

## 🎓 Learning Gatsby

Full documentation for Gatsby lives [on the website](https://next.gatsbyjs.org/).

- **For most developers, we recommend starting with our [in-depth tutorial for creating a site with Gatsby](https://next.gatsbyjs.org/tutorial/).** It starts with zero assumptions about your level of ability and walks through every step of the process.

- **To dive straight into code samples head [to our documentation](https://next.gatsbyjs.org/docs/).** In particular, check out the “Guides”, API reference, and “Advanced Tutorials” sections in the sidebar.

We welcome suggestions for improving our docs. See the [“how to contribute”](https://next.gatsbyjs.org/docs/how-to-contribute/) documentation for more details.

**Start Learning Gatsby: [Follow the Tutorial](https://next.gatsbyjs.org/tutorial/) · [Read the Docs](https://next.gatsbyjs.org/docs/)**

## 💼 Migration Guides

Already have a Gatsby site? These handy guides will help you add the improvements of Gatsby v2 to your site without starting from scratch!

- [Migrate a Gatsby site from v1 to v2](https://next.gatsbyjs.org/docs/migrating-from-v1-to-v2/)
- Still on v0? Start here: [Migrate a Gatsby site from v0 to v1](https://next.gatsbyjs.org/docs/migrating-from-v0-to-v1/)

## 🤝 How to Contribute

Whether you're helping us fix bugs, improve the docs, or spread the word, we'd love to have you as part of the Gatsby community! :muscle::purple_heart:

Check out our [contributor onboarding docs](https://next.gatsbyjs.org/docs/how-to-contribute/) for ideas on contributing and setup steps for getting our repos up and running on your local machine.

[**Read the Contributing Guide**](https://next.gatsbyjs.org/docs/how-to-contribute/)

### Code of Conduct

Gatsby is dedicated to building a welcoming, diverse, safe community. We expect everyone participating in the Gatsby community to abide by our [Code of Conduct](https://next.gatsbyjs.org/docs/code-of-conduct/). Please read it. Please follow it. In the Gatsby community, we work hard to build each other up and create amazing things together. 💪💜

[**Read the Code of Conduct**](https://next.gatsbyjs.org/docs/code-of-conduct/)

### A note on how this repository is organized

This repository is a [monorepo](https://trunkbaseddevelopment.com/monorepos/) managed using [Lerna](https://github.com/lerna/lerna). This means there are [multiple packages](/packages) managed in this codebase, even though we publish them to NPM as separate packages.

### Contributing to Gatsby v1

We are currently only accepting bug fixes for Gatsby v1. No new features will be accepted.

## 💜 Thanks to Our Contributors and Sponsors

Thanks to our many contributors and sponsors as well as the companies sponsoring
our testing and hosting infrastructure: [Travis CI](https://travis-ci.com/), [Appveyor](https://www.appveyor.com/), and [Netlify](https://www.netlify.com/).
