"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

const _ = require(`lodash`);

const _require = require(`graphql`),
      GraphQLObjectType = _require.GraphQLObjectType,
      GraphQLNonNull = _require.GraphQLNonNull,
      GraphQLID = _require.GraphQLID,
      GraphQLList = _require.GraphQLList;

const tracer = require(`opentracing`).globalTracer();

const apiRunner = require(`../utils/api-runner-node`);

const _require2 = require(`./infer-graphql-type`),
      inferObjectStructureFromNodes = _require2.inferObjectStructureFromNodes;

const _require3 = require(`./infer-graphql-input-fields-from-fields`),
      inferInputObjectStructureFromFields = _require3.inferInputObjectStructureFromFields;

const _require4 = require(`./infer-graphql-input-fields`),
      inferInputObjectStructureFromNodes = _require4.inferInputObjectStructureFromNodes;

const _require5 = require(`./node-interface`),
      nodeInterface = _require5.nodeInterface;

const _require6 = require(`../redux`),
      getNodes = _require6.getNodes,
      getNode = _require6.getNode,
      getNodeAndSavePathDependency = _require6.getNodeAndSavePathDependency;

const _require7 = require(`../redux/actions/add-page-dependency`),
      createPageDependency = _require7.createPageDependency;

const _require8 = require(`./types/type-file`),
      setFileNodeRootType = _require8.setFileNodeRootType;

const _require9 = require(`./data-tree-utils`),
      clearTypeExampleValues = _require9.clearTypeExampleValues;

const nodesCache = new Map();

module.exports =
/*#__PURE__*/
function () {
  var _ref = (0, _asyncToGenerator2.default)(function* ({
    parentSpan
  }) {
    const spanArgs = parentSpan ? {
      childOf: parentSpan
    } : {};
    const span = tracer.startSpan(`build schema`, spanArgs);

    const types = _.groupBy(getNodes().filter(node => node.internal && !node.internal.ignoreType), node => node.internal.type);

    const processedTypes = {};
    clearTypeExampleValues(); // Reset stored File type to not point to outdated type definition

    setFileNodeRootType(null);

    function createNodeFields(type) {
      const defaultNodeFields = {
        id: {
          type: new GraphQLNonNull(GraphQLID),
          description: `The id of this node.`
        },
        parent: {
          type: nodeInterface,
          description: `The parent of this node.`,

          resolve(node, a, context) {
            return getNodeAndSavePathDependency(node.parent, context.path);
          }

        },
        children: {
          type: new GraphQLList(nodeInterface),
          description: `The children of this node.`,

          resolve(node, a, {
            path
          }) {
            return node.children.map(id => getNodeAndSavePathDependency(id, path));
          }

        } // Create children fields for each type of children e.g.
        // "childrenMarkdownRemark".

      };

      const childNodesByType = _(type.nodes).flatMap(({
        children
      }) => children.map(getNode)).groupBy(node => node.internal ? _.camelCase(node.internal.type) : undefined).value();

      Object.keys(childNodesByType).forEach(childNodeType => {
        // Does this child type have one child per parent or multiple?
        const maxChildCount = _.maxBy(_.values(_.groupBy(childNodesByType[childNodeType], c => c.parent)), g => g.length).length;

        if (maxChildCount > 1) {
          defaultNodeFields[_.camelCase(`children ${childNodeType}`)] = {
            type: new GraphQLList(processedTypes[childNodeType].nodeObjectType),
            description: `The children of this node of type ${childNodeType}`,

            resolve(node, a, {
              path
            }) {
              const filteredNodes = node.children.map(id => getNode(id)).filter(({
                internal
              }) => _.camelCase(internal.type) === childNodeType); // Add dependencies for the path

              filteredNodes.forEach(n => createPageDependency({
                path,
                nodeId: n.id
              }));
              return filteredNodes;
            }

          };
        } else {
          defaultNodeFields[_.camelCase(`child ${childNodeType}`)] = {
            type: processedTypes[childNodeType].nodeObjectType,
            description: `The child of this node of type ${childNodeType}`,

            resolve(node, a, {
              path
            }) {
              const childNode = node.children.map(id => getNode(id)).find(({
                internal
              }) => _.camelCase(internal.type) === childNodeType);

              if (childNode) {
                // Add dependencies for the path
                createPageDependency({
                  path,
                  nodeId: childNode.id
                });
                return childNode;
              }

              return null;
            }

          };
        }
      });
      const inferredFields = inferObjectStructureFromNodes({
        nodes: type.nodes,
        types: _.values(processedTypes),
        ignoreFields: Object.keys(type.fieldsFromPlugins)
      });
      return Object.assign({}, defaultNodeFields, inferredFields, type.fieldsFromPlugins);
    }

    function createType(_x2, _x3) {
      return _createType.apply(this, arguments);
    } // Create node types and node fields for nodes that have a resolve function.


    function _createType() {
      _createType = (0, _asyncToGenerator2.default)(function* (nodes, typeName) {
        const intermediateType = {};
        intermediateType.name = typeName;
        intermediateType.nodes = nodes;
        const fieldsFromPlugins = yield apiRunner(`setFieldsOnGraphQLNodeType`, {
          type: intermediateType,
          traceId: `initial-setFieldsOnGraphQLNodeType`,
          parentSpan: span
        });

        const mergedFieldsFromPlugins = _.merge(...fieldsFromPlugins);

        const inferredInputFieldsFromPlugins = inferInputObjectStructureFromFields({
          fields: mergedFieldsFromPlugins
        });
        const gqlType = new GraphQLObjectType({
          name: typeName,
          description: `Node of type ${typeName}`,
          interfaces: [nodeInterface],
          fields: () => createNodeFields(proccesedType),
          isTypeOf: value => value.internal.type === typeName
        });
        const inferedInputFields = inferInputObjectStructureFromNodes({
          nodes,
          typeName
        });

        const filterFields = _.merge({}, inferedInputFields.inferredFields, inferredInputFieldsFromPlugins.inferredFields);

        const proccesedType = Object.assign({}, intermediateType, {
          fieldsFromPlugins: mergedFieldsFromPlugins,
          nodeObjectType: gqlType,
          node: {
            name: typeName,
            type: gqlType,
            args: filterFields,

            resolve(a, args, context) {
              const runSift = require(`./run-sift`);

              let latestNodes;

              if (process.env.NODE_ENV === `production` && nodesCache.has(typeName)) {
                latestNodes = nodesCache.get(typeName);
              } else {
                latestNodes = _.filter(getNodes(), n => n.internal.type === typeName);
                nodesCache.set(typeName, latestNodes);
              }

              if (!_.isObject(args)) {
                args = {};
              }

              return runSift({
                args: {
                  filter: Object.assign({}, args)
                },
                nodes: latestNodes,
                path: context.path ? context.path : ``,
                typeName: typeName,
                type: gqlType
              });
            }

          }
        });
        processedTypes[_.camelCase(typeName)] = proccesedType; // Special case to construct linked file type used by type inferring

        if (typeName === `File`) {
          setFileNodeRootType(gqlType);
        }
      });
      return _createType.apply(this, arguments);
    }

    yield Promise.all(_.map(types, createType));
    span.finish();
    return processedTypes;
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
}();
//# sourceMappingURL=build-node-types.js.map