import { Machine as XSMachine, Event, EventObject, DefaultContext, StateSchema } from './types';
import { State } from './State';
export declare type StateListener = <TContext = DefaultContext>(state: State<TContext>) => void;
export declare type ContextListener<TContext = DefaultContext> = (context: TContext, prevContext: TContext | undefined) => void;
export declare type EventListener = (event: EventObject) => void;
export declare type Listener = () => void;
export interface Clock {
    setTimeout(fn: (...args: any[]) => void, timeout: number): number;
    clearTimeout(id: number): void;
}
export interface SimulatedClock extends Clock {
    start(speed: number): void;
    increment(ms: number): void;
    set(ms: number): void;
}
interface InterpreterOptions {
    clock: Clock;
    logger: (...args: any[]) => void;
    parent?: Interpreter<any, any, any>;
}
export declare class SimulatedClock implements SimulatedClock {
    private timeouts;
    private _now;
    private _id;
    now(): number;
    private getId;
    setTimeout(fn: (...args: any[]) => void, timeout: number): number;
    clearTimeout(id: number): void;
    private flushTimeouts;
}
export declare class Interpreter<TContext, TStateSchema extends StateSchema = any, TEvents extends EventObject = EventObject> {
    machine: XSMachine<TContext, TStateSchema, TEvents>;
    static defaultOptions: InterpreterOptions;
    state: State<TContext, TEvents>;
    eventQueue: TEvents[];
    delayedEventsMap: Record<string, number>;
    activitiesMap: Record<string, any>;
    listeners: Set<StateListener>;
    contextListeners: Set<ContextListener<TContext>>;
    stopListeners: Set<Listener>;
    doneListeners: Set<StateListener>;
    eventListeners: Set<EventListener>;
    sendListeners: Set<EventListener>;
    clock: Clock;
    logger: (...args: any[]) => void;
    initialized: boolean;
    parent?: Interpreter<any>;
    private children;
    constructor(machine: XSMachine<TContext, TStateSchema, TEvents>, options?: Partial<InterpreterOptions>);
    static interpret: typeof interpret;
    /**
     * The initial state of the statechart.
     */
    readonly initialState: State<TContext, TEvents>;
    private update;
    onTransition(listener: StateListener): Interpreter<TContext>;
    onEvent(listener: EventListener): Interpreter<TContext>;
    onSend(listener: EventListener): Interpreter<TContext>;
    onChange(listener: ContextListener<TContext>): Interpreter<TContext>;
    onStop(listener: Listener): Interpreter<TContext>;
    onDone(listener: StateListener): Interpreter<TContext>;
    /**
     * Removes a listener.
     * @param listener The listener to remove
     */
    off(listener: StateListener): Interpreter<TContext>;
    init: (initialState?: State<TContext, TEvents>) => Interpreter<TContext, any, EventObject>;
    start(initialState?: State<TContext, TEvents>): Interpreter<TContext>;
    stop(): Interpreter<TContext>;
    send: (event: Event<TEvents>) => State<TContext, TEvents>;
    private forward;
    private defer;
    private cancel;
    private exec;
    private spawn;
    private flushEventQueue;
}
export declare function interpret<TContext = DefaultContext, TStateSchema extends StateSchema = any, TEvents extends EventObject = EventObject>(machine: XSMachine<TContext, TStateSchema, TEvents>, options?: Partial<InterpreterOptions>): Interpreter<TContext, TStateSchema, TEvents>;
export {};
