"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("./constants");
var utils_1 = require("./utils");
function stateValuesEqual(a, b) {
    if (a === b) {
        return true;
    }
    var aKeys = utils_1.keys(a);
    var bKeys = utils_1.keys(b);
    return (aKeys.length === bKeys.length &&
        aKeys.every(function (key) { return stateValuesEqual(a[key], b[key]); }));
}
exports.stateValuesEqual = stateValuesEqual;
var State = /** @class */ (function () {
    function State(value, context, historyValue, history, actions, activities, data, 
    /**
     * Internal event queue
     */
    events, tree) {
        if (actions === void 0) { actions = []; }
        if (activities === void 0) { activities = constants_1.EMPTY_ACTIVITY_MAP; }
        if (data === void 0) { data = {}; }
        if (events === void 0) { events = []; }
        this.value = value;
        this.context = context;
        this.historyValue = historyValue;
        this.history = history;
        this.actions = actions;
        this.activities = activities;
        this.data = data;
        this.events = events;
        Object.defineProperty(this, 'tree', {
            value: tree,
            enumerable: false
        });
    }
    State.from = function (stateValue, context) {
        if (stateValue instanceof State) {
            if (stateValue.context !== context) {
                return new State(stateValue.value, context, stateValue.historyValue, stateValue.history, [], stateValue.activities, {}, [], stateValue.tree);
            }
            return stateValue;
        }
        return new State(stateValue, context, undefined, undefined, [], undefined, undefined, []);
    };
    State.inert = function (stateValue, context) {
        if (stateValue instanceof State) {
            if (!stateValue.actions.length) {
                return stateValue;
            }
            return new State(stateValue.value, context, stateValue.historyValue, stateValue.history, undefined, stateValue.activities, undefined, undefined, stateValue.tree);
        }
        return State.from(stateValue, context);
    };
    Object.defineProperty(State.prototype, "nextEvents", {
        get: function () {
            if (!this.tree) {
                return [];
            }
            return this.tree.nextEvents;
        },
        enumerable: true,
        configurable: true
    });
    State.prototype.toStrings = function (value, delimiter) {
        var _this = this;
        if (value === void 0) { value = this.value; }
        if (delimiter === void 0) { delimiter = '.'; }
        if (typeof value === 'string') {
            return [value];
        }
        var valueKeys = utils_1.keys(value);
        return valueKeys.concat.apply(valueKeys, __spread(valueKeys.map(function (key) {
            return _this.toStrings(value[key]).map(function (s) { return key + delimiter + s; });
        })));
    };
    State.prototype.matches = function (parentStateValue) {
        return utils_1.matchesState(parentStateValue, this.value);
    };
    Object.defineProperty(State.prototype, "changed", {
        get: function () {
            if (!this.history) {
                return undefined;
            }
            return (!!this.actions.length ||
                (typeof this.history.value !== typeof this.value
                    ? true
                    : typeof this.value === 'string'
                        ? this.value !== this.history.value
                        : stateValuesEqual(this.value, this.history.value)));
        },
        enumerable: true,
        configurable: true
    });
    return State;
}());
exports.State = State;
