import { Event, StateValue, Action, StateTransition, MachineOptions, EventObject, HistoryValue, DefaultContext, StateNodeDefinition, AssignAction, DelayedTransitionDefinition, ActivityDefinition, Delay, StateTypes, StateNodeConfig, StateSchema, TransitionsDefinition, StateNodesConfig, AnyEvent } from './types';
import { State } from './State';
declare class StateNode<TContext = DefaultContext, TStateSchema extends StateSchema = any, TEvents extends AnyEvent<EventObject> = AnyEvent<EventObject>> {
    private _config;
    options: Readonly<MachineOptions<TContext, TEvents>>;
    /**
     * The initial extended state
     */
    context?: Readonly<TContext> | undefined;
    key: string;
    id: string;
    type: StateTypes;
    path: string[];
    initial?: keyof TStateSchema['states'];
    parallel: boolean;
    transient: boolean;
    states: StateNodesConfig<TContext, TStateSchema, TEvents>;
    history: false | 'shallow' | 'deep';
    onEntry: Array<Action<TContext>>;
    onExit: Array<Action<TContext>>;
    activities: Array<ActivityDefinition<TContext>>;
    strict: boolean;
    parent?: StateNode<TContext>;
    machine: StateNode<TContext>;
    data?: TStateSchema extends {
        data: infer D;
    } ? D : any;
    delimiter: string;
    order: number;
    private __cache;
    private idMap;
    constructor(_config: StateNodeConfig<TContext, TStateSchema, TEvents>, options?: Readonly<MachineOptions<TContext, TEvents>>, 
    /**
     * The initial extended state
     */
    context?: Readonly<TContext> | undefined);
    withConfig(options: MachineOptions<TContext, TEvents>, context?: TContext): StateNode<TContext, TStateSchema, TEvents>;
    withContext(context: TContext): StateNode<TContext, TStateSchema, TEvents>;
    readonly definition: StateNodeDefinition<TContext, TStateSchema, TEvents>;
    readonly config: StateNodeConfig<TContext, TStateSchema, TEvents>;
    readonly on: TransitionsDefinition<TContext, TEvents>;
    readonly after: Array<DelayedTransitionDefinition<TContext, TEvents>>;
    getStateNodes(state: StateValue | State<TContext, TEvents>): Array<StateNode<TContext>>;
    handles(event: Event<TEvents>): boolean;
    private transitionLeafNode;
    private transitionCompoundNode;
    private transitionParallelNode;
    _transition(stateValue: StateValue, state: State<TContext, TEvents>, event: TEvents, context?: TContext): StateTransition<TContext>;
    private next;
    private readonly tree;
    private nodesFromChild;
    private getStateTree;
    private escapes;
    private evaluateGuard;
    readonly delays: Delay[];
    private getActions;
    private resolveAction;
    private resolveActivity;
    private getActivities;
    transition(state: StateValue | State<TContext, TEvents>, event: Event<TEvents>, context?: TContext): State<TContext, TEvents>;
    private resolveTransition;
    static updateContext<TContext, TEvents extends EventObject>(context: TContext, event: TEvents | undefined, assignActions: Array<AssignAction<TContext, TEvents>>): TContext;
    private ensureValidPaths;
    getStateNode(stateKey: string): StateNode<TContext>;
    getStateNodeById(stateId: string): StateNode<TContext>;
    getStateNodeByPath(statePath: string | string[]): StateNode<TContext>;
    resolve(stateValue: StateValue): StateValue;
    private readonly resolvedStateValue;
    private getResolvedPath;
    private readonly initialStateValue;
    getInitialState(stateValue: StateValue, context?: TContext): State<TContext, TEvents>;
    readonly initialState: State<TContext, TEvents>;
    readonly target: StateValue | undefined;
    getStates(stateValue: StateValue): Array<StateNode<TContext>>;
    /**
     * Returns the leaf nodes from a state path relative to this state node.
     *
     * @param relativeStateId The relative state path to retrieve the state nodes
     * @param history The previous state to retrieve history
     * @param resolve Whether state nodes should resolve to initial child state nodes
     */
    getRelativeStateNodes(relativeStateId: string | string[], historyValue?: HistoryValue, resolve?: boolean): Array<StateNode<TContext>>;
    readonly initialStateNodes: Array<StateNode<TContext>>;
    /**
     * Retrieves state nodes from a relative path to this state node.
     *
     * @param relativePath The relative path from this state node
     * @param historyValue
     */
    getFromRelativePath(relativePath: string[], historyValue?: HistoryValue): Array<StateNode<TContext>>;
    static updateHistoryValue(hist: HistoryValue, stateValue: StateValue): HistoryValue;
    historyValue(relativeStateValue?: StateValue | undefined): HistoryValue | undefined;
    /**
     * Resolves to the historical value(s) of the parent state node,
     * represented by state nodes.
     *
     * @param historyValue
     */
    private resolveHistory;
    readonly stateIds: string[];
    readonly events: Array<TEvents['type']>;
    readonly ownEvents: Array<TEvents['type']>;
    private formatTransition;
    private formatTransitions;
}
export { StateNode };
