"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("./types");
var actionTypes = require("./actionTypes");
exports.actionTypes = actionTypes;
var utils_1 = require("./utils");
function toEventObject(event
// id?: TEvents['type']
) {
    if (typeof event === 'string' || typeof event === 'number') {
        var eventObject = { type: event };
        // if (id !== undefined) {
        //   eventObject.id = id;
        // }
        return eventObject;
    }
    return event;
}
exports.toEventObject = toEventObject;
function getActionFunction(actionType, actionFunctionMap) {
    if (!actionFunctionMap) {
        return undefined;
    }
    var actionReference = actionFunctionMap[actionType];
    if (!actionReference) {
        return undefined;
    }
    if (typeof actionReference === 'function') {
        return actionReference;
    }
    return actionReference.exec;
}
function toActionObject(action, actionFunctionMap) {
    var actionObject;
    if (typeof action === 'string' || typeof action === 'number') {
        actionObject = {
            type: action,
            exec: getActionFunction(action, actionFunctionMap)
        };
    }
    else if (typeof action === 'function') {
        actionObject = {
            type: action.name,
            exec: action
        };
    }
    else {
        var exec = getActionFunction(action.type, actionFunctionMap);
        return exec
            ? __assign({}, action, { exec: exec }) : action;
    }
    Object.defineProperty(actionObject, 'toString', {
        value: function () { return actionObject.type; },
        enumerable: false
    });
    return actionObject;
}
exports.toActionObject = toActionObject;
function toActivityDefinition(action) {
    var actionObject = toActionObject(action);
    return __assign({ id: typeof action === 'string' ? action : actionObject.id }, actionObject, { type: actionObject.type });
}
exports.toActivityDefinition = toActivityDefinition;
exports.toActionObjects = function (action, actionFunctionMap) {
    if (!action) {
        return [];
    }
    var actions = Array.isArray(action) ? action : [action];
    return actions.map(function (subAction) { return toActionObject(subAction, actionFunctionMap); });
};
function raise(eventType) {
    return {
        type: actionTypes.raise,
        event: eventType
    };
}
exports.raise = raise;
function send(event, options) {
    return {
        to: options ? options.target : undefined,
        type: actionTypes.send,
        event: toEventObject(event),
        delay: options ? options.delay : undefined,
        id: options && options.id !== undefined
            ? options.id
            : utils_1.getEventType(event)
    };
}
exports.send = send;
function sendParent(event, options) {
    return send(event, __assign({}, options, { target: types_1.SpecialTargets.Parent }));
}
exports.sendParent = sendParent;
function log(expr, label) {
    return {
        type: actionTypes.log,
        label: label,
        expr: expr
    };
}
exports.log = log;
exports.cancel = function (sendId) {
    return {
        type: actionTypes.cancel,
        sendId: sendId
    };
};
function start(activity) {
    var activityDef = toActivityDefinition(activity);
    return {
        type: types_1.ActionTypes.Start,
        activity: activityDef,
        exec: undefined
    };
}
exports.start = start;
function stop(activity) {
    var activityDef = toActivityDefinition(activity);
    return {
        type: types_1.ActionTypes.Stop,
        activity: activityDef,
        exec: undefined
    };
}
exports.stop = stop;
exports.assign = function (assignment) {
    return {
        type: actionTypes.assign,
        assignment: assignment
    };
};
function isActionObject(action) {
    return typeof action === 'object' && 'type' in action;
}
exports.isActionObject = isActionObject;
function after(delay, id) {
    var idSuffix = id ? "#" + id : '';
    return types_1.ActionTypes.After + "(" + delay + ")" + idSuffix;
}
exports.after = after;
function done(id) {
    return types_1.ActionTypes.DoneState + "." + id;
}
exports.done = done;
function invoke(invokeConfig, options) {
    if (typeof invokeConfig === 'string') {
        return __assign({ id: invokeConfig, src: invokeConfig, type: types_1.ActionTypes.Invoke }, options);
    }
    return invokeConfig;
}
exports.invoke = invoke;
